/**
 *
 * Copyright (C) 2013-2014, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */
#ifndef __MLNX_HWDEV_H_
#define __MLNX_HWDEV_H_

#ifndef __KERNEL__
#define __user 
#endif

#include <linux/i2c.h>

#define I2C_BUS_TYPE            0
#define LPC_BUS_TYPE            1
#define CPLD_MASK_NOP8          0xFF
#define CPLD_MASK_NOP16         0xFFFF
#define CPLD_MASK_NOP32         0xFFFFFFFF
#define CPLD_NO_DEV             0xFF

#ifndef max2
 #define max2(a,b) ( { __typeof__ (a) _a = (a); __typeof__ (b) _b = (b); _a > _b ? _a : _b; } )
#endif

#ifndef min2
 #define min2(a,b) ( { __typeof__ (a) _a = (a); __typeof__ (b) _b = (b); _a < _b ? _a : _b; } )
#endif

#ifndef max3
 #define max3(a,b,c) ( { __typeof__ (a) _a = (a); __typeof__ (b) _b = (b); __typeof__ (c) _c = (c); \
			(a>b)?((a>c)?a:c):((b>c)?b:c) } )
#endif

#ifndef max4
 #define max4(a,b,c,d) ( max2(max2(a,b),max2(c,d)))
#endif

#ifndef min4
 #define min4(a,b,c,d) ( min2(min2(a,b),min2(c,d)))
#endif

#define _B_NOLOCK 0
#define _B_LOCK   1
#define _B_I2C    1
#define _B_LPC    2
#define _B_ALL    3

struct mlnx_i2c_dev_params {
	struct i2c_adapter *adap;      /* Adapter handle (per virtual bus) */
	int bus_id;                    /* Bus id to where device is attached */
	int addr_space;                /* Address width of i2c device */
	__u16 addr;	               /* Slave address */
};

struct mlnx_lpc_dev_params {
	__u16 base;                     /* Low Pin Count (LPC) bus access base address */
	__u16 size;                     /* Size of mapped address space */
	int reserved;
};

union hw_dev {                      /* Access parameters */
	struct mlnx_i2c_dev_params i2c_params;
	struct mlnx_lpc_dev_params lpc_params;
};

struct bus_reset_block {
	union hw_dev hwdev;
	__u8 offset;          /* reset offset access */
	__u8 offset_wp;       /* reset write protect offset access */
	__u8 toggle_off_mask; /* reset write mask */
	__u8 toggle_on_mask;  /* reset write mask */
};

typedef int (*rw_bus_access_func)(union hw_dev *hw_data,
				  __u8 from_range,
				  __u8 to_range,
				  __u8 rw_flag,
				  __u8 *p_data,
				  __u8 lock_flag);

typedef int (*rw_bus_reset_func)(struct bus_reset_block *bus_rst,
				 __u8 lock_flag);

int i2c_bus_access_func(union hw_dev *hw_data,
			__u8 from_range,
			__u8 to_range,
			__u8 rw_flag,
			__u8 *p_data,
			__u8 lock_flag);

int lpc_bus_access_func(union hw_dev *hw_data, 
			__u8 from_range,
			__u8 to_range,
			__u8 rw_flag,
			__u8 *p_data,
			__u8 lock_flag);

int i2c_bus_reset_func(struct bus_reset_block *bus_rst,
		       __u8 lock_flag);

int lpc_bus_reset_func(struct bus_reset_block *bus_rst,
		       __u8 lock_flag);

void _lock_bus(__u8 lock_unlock_flag);

void _unlock_bus(__u8 lock_unlock_flag);

#endif /*__MLNX_HWDEV_H_*/
