/**
 *
 * Copyright (C) 2011-2014, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */
#ifndef __MLNX_IF_H_
#define __MLNX_IF_H_

#ifndef __KERNEL__
#define __user 
#endif

#include "mlnx_hwdev.h"
#include "mlnx_irq.h"
#include "mlnx_watchdog.h"
#include "mlnx_bsp.h"

#define MLNX_IF_DEVICE_NAME    "/dev/mlnxif"
#define MLNX_IF_DEVICE_NAME_UNBLOCKED "/dev/mlnxif_unb"
#define MLNX_IF_DEVICE_MAJOR   243
#define MLNX_IF_DEVICE_MAJOR_UNBLOCKED 244
#define SMBUS_BLOCK_MAX        256
#define I2C_BLOCK_MAX          132
#define LPCI2C_DATA_REG_SZ     36   /* Same as in lpci2c.h */
#define I2C_CHUNK_MAX          128
#define I2C_MAX_RW_MSGS          2
#define PCA954x_SWITCH_NUM     256 /* from 1 to 255, 0 - not exist */
#define PCA954x_SWITCH_NONE      0 /* not exist */
#define I2C_BUS_NONE             0 /* not exist */
#define BASE_VIRT_BUS            2
#define LEGS_NUM                 8
#define MAX_SW_INREQ_NUM         4
#define MAX_DEV_INREQ_NUM        8
#define MAX_I2C_VBUS_NUM       (PCA954x_SWITCH_NUM * LEGS_NUM)
#define MAX_I2C_DEV_NUM        (MAX_I2C_VBUS_NUM * MAX_DEV_INREQ_NUM)
#define MAX_I2C_GPIO_INREQ_NUM   4
#define MAX_I2C_GPIO_BUSES       4
#define MAX_GPIO_NUM            40
#define MAX_GPIO_NUM            40
#define SWITCH_NAME_SIZE         8
#define RESET_FLAG_ADD_DEV      0x10
#define RESET_FLAG_REMOVE_DEV   0x20
#define RESET_FLAG_RESET_SWITCH 0x40
#define DEV_NOT_EXIST           0x0
#define DEV_EXIST_OK            0x1
#define DEV_BAD_GO_BIT          0x2
#define DEV_FAILED_PROBE        0x4
#define DEV_FAILED_HCR_ACCESS   0x8
#define CR_SPACE_SIZE           0xFFE78
#define CR_SPACE_CHUNK          0x800
#define CPLD_SUBSCRIBE          0
#define CPLD_UNSUBSCRIBE        1
#define CPLD_STATUS_UNCHANGED   0
#define CPLD_STATUS_CHANGED     1
#define CPLD_EVENT_NOT_OCCURRED 0
#define CPLD_EVENT_OCCURRED     1
#define CPLD_EVENT_MASK         0
#define CPLD_EVENT_ENABLE       1
#define CPLD_EVENT_CLEAR        1
#define CPLD_INTERRUPT_ENABLE   0
#define CPLD_INTERRUPT_MASK     1
#define CARD_TYPE_NONE          0
#define CARD_TYPE_MGMT          1
#define CARD_TYPE_MODULE        2
#define CARD_TYPE_TOR           3
#define CARD_TYPE_PORT          4
#define MAX_BUS_TYPES           2 /* Supported devices attached to LPC or I2C buses */
#define DEV_INT_CAP_MAX         9 /* Maximum number of interrupt capable devices on one module */
#define DEV_TERM_INT_CAP_MAX    4 /* Maximum number of interrupt capable termination devices on one module */
#define DEV_INDEX_NONE        255
#define MAIN_MODULE_HANDLE   0xFF
#define _E_AGGR            0x8000
#define _E_INV             0x4000
#define _E_INV_S           0x2000
#define _E_INV_M           0x1000
#define _E_AVAIL           0x0100
#define _E_UNSET           0x0200
#define _E_TERM            0x0400
#define _E_MASK_ID         0x00FF
#define _E_MASK_FEAT       0xFF00
#define PPC_GPIO0_BASE_NR     224
#define PPC_GPIO1_BASE_NR     192

#define SET_SWITCH_ID(ind, bus, addr) ( ((ind) << 24) | (((bus) << 8) & 0x000fff00) | (addr & 0x7f) )
#define GET_BUS_ID_BY_SWITCH_ID(switch_id) ( (switch_id >> 8) & 0x00000fff)
#define GET_SWITCH_INDEX_BY_SWITCH_ID(switch_id) ((switch_id & 0xff000000) >> 24)

#ifndef ARRAY_SIZE
	#define ARRAY_SIZE(x) ( (sizeof(x) / sizeof((x)[0])) )
#endif

#ifndef FIELD_SIZEOF
	#define FIELD_SIZEOF(t, f) (sizeof(((t*)0)->f))
#endif

/*
Interrupt code			Value		Priority	Mgmt	Spine	Leaf	Fan	PS
BOARD				0x80000000	High		Y	N	N	N	N
HA_LOCAL								N	N	N	N
HA_REMOTE								N	N	N	N
HA_FROM_OTHER_CPU_MASTER	0x40000000	High		Y	N	N	N	N
HA_ FROM_OTHER_CPU_RESET_ACK	0x20000000	High		Y	N	N	N	N
HA_FROM_OTHER_CPU_INT_N		0x10000000	High		Y	N	N	N	N
HA_FROM_OTHER_CPU_PRESENT_N	0x08000000	High		Y	N	N	N	N
HA_FROM_OTHER_CPU_ACTIVE	0x04000000	High		Y	N	N	N	N
HA_REMOTE_FROM_LOCAL_AGGR	0x02000000	High		Y	N	N	N	N
HA_REMOTE_ROLE_CHANGE		0x01000000	High		Y	N	N	N	N
HA_REMOTE_ROLE_BOTH_CPLD_MASTER	0x00800000	High		Y	N	N	N	N
HA_REMOTE_SW_INT		0x00400000	High		Y	N	N	N	N
HA_OTHER_CPU_PRESENT		0x00200000	High		Y	N	N	N	N
HA_OTHER_CPU_WD_EXPIRED		0x00100000	High		Y	N	N	N	N
HA_RESET_IND1			0x00080000	High		Y	N	N	N	N
HA_RESET_IND2			0x00040000	High		Y	N	N	N	N
HA_SUPPORT_IND1			0x00020000	High		Y	N	N	N	N
HA_SUPPORT_IND2			0x00010000	High		Y	N	N	N	N
PRESENT				0x00008000	Medium		Y	N	N	N	N
FAN_SPEED			0x00004000	Medium		Y	N	N	N	N
AC_FAIL				0x00002000	Medium		Y	N	N	N	N
DC_FAIL				0x00001000	Medium		Y	N	N	N	N
PS_THERM			0x00000800	Medium		Y	N	N	N	N
MONITOR_THERM			0x00000400	Medium		N	Y	Y	N	N
MONITOR_INT			0x00000200	Medium		N	Y	Y	N	N
SX_THERM_SDWN			0x00000100	Medium		N	Y	Y	N	N
SX_HEALTH			0x00000080	Medium		N	Y	Y	N	N
PWRON				0x00000040			N	Y	N	N	N
PWRGOOD_ALL			0x00000020	Medium		N	N	Y	N	N
PWRGOOD_12V			0x00000010	Medium		N	N	Y	N	N
SX_FAN_PRSNT 			0x00000008	Medium		N	Y	N	N	N
SX_FAN_SPEED			0x00000004	Medium		N	Y	N	N	N
SX_FAN_ERR			0x00000002	Medium		N	Y	N	N	N
SX_THERM_WARN			0x00000001	Low		N	Y	Y	N	N
						
DIGEST								MDIGEST	SDIGEST	LDIGEST	FDIGEST	PDIGEST
*/
typedef enum irq_cause {
        BOARD,
        IRQ_CAUSE_FIRST = BOARD,
        HA_LOCAL,
        HA_REMOTE,
        HA_FROM_OTHER_CPU_MASTER,
        HA_FROM_OTHER_CPU_RESET_ACK,
        HA_FROM_OTHER_CPU_INT_N,
        HA_FROM_OTHER_CPU_PRESENT_N,
        HA_FROM_OTHER_CPU_ACTIVE,
        HA_REMOTE_FROM_LOCAL_AGGR,
        HA_REMOTE_ROLE_CHANGE,
        HA_REMOTE_ROLE_BOTH_CPLD_MASTER,
        HA_REMOTE_SW_INT,
        HA_OTHER_CPU_PRESENT,
        HA_OTHER_CPU_WD_EXPIRED,
        HA_RESET_IND1,
        HA_RESET_IND2,
        HA_SUPPORT_IND1,
        HA_SUPPORT_IND2,
        PRESENT,
        FAN_SPEED,
        AC_FAIL,
        DC_FAIL,
        PS_THERM,
        MONITOR_THERM,
        MONITOR_INT,
        SX_THERM_SDWN,
        SX_HEALTH,
        PWRON,
        PWRGOOD_ALL,
        PWRGOOD_12V,
        SX_FAN_PRSNT,
        SX_FAN_SPEED,
        SX_FAN_ERR,
        SX_THERM_WARN,
        IRQ_CAUSE_LAST = SX_THERM_WARN,
        IRQ_CAUSE_NONE = IRQ_CAUSE_LAST + 1,
} irq_cause_e;

typedef enum irq_value {
        _V_BOARD                           = 0x80000000,
        _V_HA_LOCAL                        = 0x00000000,
        _V_HA_REMOTE                       = 0x00000000,
        _V_HA_FROM_OTHER_CPU_MASTER        = 0x40000000,
        _V_HA_FROM_OTHER_CPU_RESET_ACK     = 0x20000000,
        _V_HA_FROM_OTHER_CPU_INT_N         = 0x10000000,
        _V_HA_FROM_OTHER_CPU_PRESENT_N     = 0x08000000,
        _V_HA_FROM_OTHER_CPU_ACTIVE        = 0x04000000,
        _V_HA_REMOTE_FROM_LOCAL_AGGR       = 0x02000000,
        _V_HA_REMOTE_ROLE_CHANGE           = 0x01000000,
        _V_HA_REMOTE_ROLE_BOTH_CPLD_MASTER = 0x00800000,
        _V_HA_REMOTE_SW_INT                = 0x00400000,
        _V_HA_OTHER_CPU_PRESENT            = 0x00200000,
        _V_HA_OTHER_CPU_WD_EXPIRED         = 0x00100000,
        _V_HA_RESET_IND1                   = 0x00080000,
        _V_HA_RESET_IND2                   = 0x00040000,
        _V_HA_SUPPORT_IND1                 = 0x00020000,
        _V_HA_SUPPORT_IND2                 = 0x00010000,
        _V_PRESENT                         = 0x00008000,
        _V_FAN_SPEED                       = 0x00004000,
        _V_AC_FAIL                         = 0x00002000,
        _V_DC_FAIL                         = 0x00001000,
        _V_PS_THERM                        = 0x00000800,
        _V_MONITOR_THERM                   = 0x00000400,
        _V_MONITOR_INT                     = 0x00000200,
        _V_SX_THERM_SDWN                   = 0x00000100,
        _V_SX_HEALTH                       = 0x00000080,
        _V_PWRON                           = 0x00000040,
        _V_PWRGOOD_ALL                     = 0x00000020,
        _V_PWRGOOD_12V                     = 0x00000010,
        _V_SX_FAN_PRSNT                    = 0x00000008,
        _V_SX_FAN_SPEED                    = 0x00000004,
        _V_SX_FAN_ERR                      = 0x00000002,
        _V_SX_THERM_WARN                   = 0x00000001,
} irq_value_e;

typedef enum irq_signal {
        _none_sig                  = 0x0000,
        _aggr_m_mezz_int           = 0x0001,
        _aggr_m_mezz_pg_int        = 0x0002,
        _aggr_m_mezz_main_brd_int1 = 0x0003,
        _aggr_m_mezz_main_brd_int2 = 0x0004,
        _aggr_m_mezz_main_brd_int3 = 0x0005,
        _sx_reg                    = 0x0006,
        _wd_                       = 0x0007,
        _fan_prsnt_int             = 0x0008,
        _ps_int                    = 0x0009,
        _pg_int                    = 0x000A,
        _board_int_reg_1           = 0x000B,
        _board_int_reg_2           = 0x000C,
        _pg_1_8_aux                = 0x000D,
        _pg_1_8_sx                 = 0x000E,
        _pg_3_3_aux                = 0x000F,
        _pg_3_3_sx                 = 0x0010,
        _pg_vcore                  = 0x0011,
        _pg_1_2                    = 0x0012,
        _pg_5                      = 0x0013,
        _pg_x86                    = 0x0014,
        _qsfp_oc                   = 0x0015,
        _usb_oc                    = 0x0016,
        _qsfp_int                  = 0x0017,
        _adm_int                   = 0x0018,
        _adm_term                  = 0x0019,
        _ucd_3_3_alert             = 0x001A,
        _ucd_vcore_alert           = 0x001B,
        _curr_alert_12             = 0x001C,
        _temp_sens_x49             = 0x001D,
        _temp_sens_x4a             = 0x001E,
        _temp_sens_x4c             = 0x001F,
        _asic_health0              = 0x0020,
        _asic_health1              = 0x0021,
        _asic_temp_warn            = 0x0022,
        _asic_temp_shutdown        = 0x0023,
        _asic_fan_err              = 0x0024,
        _ps1_prsnt                 = 0x0025,
        _ps2_prsnt                 = 0x0026,
        _ps3_prsnt                 = 0x0027,
        _ps4_prsnt                 = 0x0028,
        _ps5_prsnt                 = 0x0029,
        _ps6_prsnt                 = 0x002A,
        _ps7_prsnt                 = 0x002B,
        _ps8_prsnt                 = 0x002C,
        _ps9_prsnt                 = 0x002D,
        _ps10_prsnt                = 0x002E,
        _ps1_pg_fru_dc_ok          = 0x002F,
        _ps2_pg_fru_dc_ok          = 0x0030,
        _ps3_pg_fru_dc_ok          = 0x0031,
        _ps4_pg_fru_dc_ok          = 0x0032,
        _ps5_pg_fru_dc_ok          = 0x0033,
        _ps6_pg_fru_dc_ok          = 0x0034,
        _ps7_pg_fru_dc_ok          = 0x0035,
        _ps8_pg_fru_dc_ok          = 0x0036,
        _ps9_pg_fru_dc_ok          = 0x0037,
        _ps10_pg_fru_dc_ok         = 0x0038,
        _ps1_temp_alarm            = 0x0039,
        _ps2_temp_alarm            = 0x003A,
        _ps3_temp_alarm            = 0x003B,
        _ps4_temp_alarm            = 0x003C,
        _ps5_temp_alarm            = 0x003D,
        _ps6_temp_alarm            = 0x003E,
        _ps7_temp_alarm            = 0x003F,
        _ps8_temp_alarm            = 0x0040,
        _ps9_temp_alarm            = 0x0041,
        _ps10_temp_alarm           = 0x0042,
        _fan1_prsnt                = 0x0043,
        _fan2_prsnt                = 0x0044,
        _fan3_prsnt                = 0x0045,
        _fan4_prsnt                = 0x0046,
        _fan5_prsnt                = 0x0047,
        _fan6_prsnt                = 0x0048,
        _fan7_prsnt                = 0x0049,
        _fan8_prsnt                = 0x004A,
        _pw_good_0_85              = 0x004B,
        _pw_good_1_05_cpu          = 0x004C,
        _pw_good_1_05_pch          = 0x004D,
        _pw_good_ddr               = 0x004E,
        _pw_good_vcc5_a            = 0x004F,
        _ts_dimm_a                 = 0x0050,
        _ts_dimm_b                 = 0x0051,
        _x86_hot                   = 0x0052,
        _pch_hot                   = 0x0053,
        _temp_sens_mezz            = 0x0054,
        _cpu_fatal_err             = 0x0055,
        _lpc_int                   = 0x0056,
        _wd1_int                   = 0x0057,
        _wd2_int                   = 0x0058,
        _wd3_int                   = 0x0059,
        _wd4_int                   = 0x005A,
        _leaf01_prsnt              = 0x005B,
        _leaf02_prsnt              = 0x005C,
        _leaf03_prsnt              = 0x005D,
        _leaf04_prsnt              = 0x005E,
        _leaf05_prsnt              = 0x005F,
        _leaf06_prsnt              = 0x0060,
        _leaf07_prsnt              = 0x0061,
        _leaf08_prsnt              = 0x0062,
        _leaf09_prsnt              = 0x0063,
        _leaf10_prsnt              = 0x0064,
        _leaf11_prsnt              = 0x0065,
        _leaf12_prsnt              = 0x0066,
        _leaf13_prsnt              = 0x0067,
        _leaf14_prsnt              = 0x0068,
        _leaf15_prsnt              = 0x0069,
        _leaf16_prsnt              = 0x006A,
        _leaf17_prsnt              = 0x006B,
        _leaf18_prsnt              = 0x006C,
        _leaf19_prsnt              = 0x006D,
        _leaf20_prsnt              = 0x006E,
        _leaf21_prsnt              = 0x006F,
        _leaf22_prsnt              = 0x0070,
        _leaf23_prsnt              = 0x0071,
        _leaf24_prsnt              = 0x0072,
        _leaf25_prsnt              = 0x0073,
        _leaf26_prsnt              = 0x0074,
        _leaf27_prsnt              = 0x0075,
        _leaf28_prsnt              = 0x0076,
        _leaf29_prsnt              = 0x0077,
        _leaf30_prsnt              = 0x0078,
        _leaf31_prsnt              = 0x0079,
        _leaf32_prsnt              = 0x007A,
        _leaf33_prsnt              = 0x007B,
        _leaf34_prsnt              = 0x007C,
        _leaf35_prsnt              = 0x007D,
        _leaf36_prsnt              = 0x007E,
        _spine01_prsnt             = 0x007F,
        _spine02_prsnt             = 0x0080,
        _spine03_prsnt             = 0x0081,
        _spine04_prsnt             = 0x0082,
        _spine05_prsnt             = 0x0083,
        _spine06_prsnt             = 0x0084,
        _spine07_prsnt             = 0x0085,
        _spine08_prsnt             = 0x0086,
        _spine09_prsnt             = 0x0087,
        _spine10_prsnt             = 0x0088,
        _spine11_prsnt             = 0x0089,
        _spine12_prsnt             = 0x008A,
        _spine13_prsnt             = 0x008B,
        _spine14_prsnt             = 0x008C,
        _spine15_prsnt             = 0x008D,
        _spine16_prsnt             = 0x008E,
        _spine17_prsnt             = 0x008F,
        _spine18_prsnt             = 0x0090,
        _leaf01_int                = 0x0091,
        _leaf02_int                = 0x0092,
        _leaf03_int                = 0x0093,
        _leaf04_int                = 0x0094,
        _leaf05_int                = 0x0095,
        _leaf06_int                = 0x0096,
        _leaf07_int                = 0x0097,
        _leaf08_int                = 0x0098,
        _leaf09_int                = 0x0099,
        _leaf10_int                = 0x009A,
        _leaf11_int                = 0x009B,
        _leaf12_int                = 0x009C,
        _leaf13_int                = 0x009D,
        _leaf14_int                = 0x009E,
        _leaf15_int                = 0x009F,
        _leaf16_int                = 0x00A0,
        _leaf17_int                = 0x00A1,
        _leaf18_int                = 0x00A2,
        _leaf19_int                = 0x00A3,
        _leaf20_int                = 0x00A4,
        _leaf21_int                = 0x00A5,
        _leaf22_int                = 0x00A6,
        _leaf23_int                = 0x00A7,
        _leaf24_int                = 0x00A8,
        _leaf25_int                = 0x00A9,
        _leaf26_int                = 0x00AA,
        _leaf27_int                = 0x00AB,
        _leaf28_int                = 0x00AC,
        _leaf29_int                = 0x00AD,
        _leaf30_int                = 0x00AE,
        _leaf31_int                = 0x00AF,
        _leaf32_int                = 0x00B0,
        _leaf33_int                = 0x00B1,
        _leaf34_int                = 0x00B2,
        _leaf35_int                = 0x00B3,
        _leaf36_int                = 0x00B4,
        _spine01_int               = 0x00B5,
        _spine02_int               = 0x00B6,
        _spine03_int               = 0x00B7,
        _spine04_int               = 0x00B8,
        _spine05_int               = 0x00B9,
        _spine06_int               = 0x00BA,
        _spine07_int               = 0x00BB,
        _spine08_int               = 0x00BC,
        _spine09_int               = 0x00BD,
        _spine10_int               = 0x00BE,
        _spine11_int               = 0x00BF,
        _spine12_int               = 0x00C0,
        _spine13_int               = 0x00C1,
        _spine14_int               = 0x00C2,
        _spine15_int               = 0x00C3,
        _spine16_int               = 0x00C4,
        _spine17_int               = 0x00C5,
        _spine18_int               = 0x00C6,
        _leaf_aggr_int             = 0x00C7,
        _spine_fan_aggr_int        = 0x00C8,
        _leaf_aggr_prsnt           = 0x00C9,
        _spine_fan_aggr_prsnt      = 0x00CA,
        _ps_aggr_pwr_temp_prsnt    = 0x00CB,
        _mate_mgmt_prsnt           = 0x00CC,
        _ps1_pg_fru_ac_ok          = 0x00CD,
        _ps2_pg_fru_ac_ok          = 0x00CE,
        _ps3_pg_fru_ac_ok          = 0x00CF,
        _ps4_pg_fru_ac_ok          = 0x00D0,
        _ps5_pg_fru_ac_ok          = 0x00D1,
        _ps6_pg_fru_ac_ok          = 0x00D2,
        _ps7_pg_fru_ac_ok          = 0x00D3,
        _ps8_pg_fru_ac_ok          = 0x00D4,
        _ps9_pg_fru_ac_ok          = 0x00D5,
        _ps10_pg_fru_ac_ok         = 0x00D6,
        _aggr_main_brd_pg_int      = 0x00D7,
        _aggr_main_brd_int1        = 0x00D8,
        _aggr_main_brd_int2        = 0x00D9,
        _aggr_main_brd_int3        = 0x00DA,
        _temp_sens_main_brd        = 0x00DB,
        _pg_cx3                    = 0x00DC,
        _pg_12                     = 0x00DD,
        _max_sig_num               = _pg_12,
        _mark_end_sig              = 0xFEDC,
} irq_signal_e;

/* I2C related ioctl parameters */
struct mlnx_rdwr_i2c_ioctl_params {
	int bus;                       /* Bus id to which read or write request is to be sent */
	int nmsgs;                     /* Number of i2c messages to be sent within the request 
                                          (number of i2c_msg items)  */
	struct __user i2c_msg msgs[I2C_MAX_RW_MSGS]; /* Template contains the following fields:
	                                         - addr: i2c device slave address;
	                                         - flags: (f.e. I2C_M_RD - for read from slave; 
	                                                        I2C_M_TEN - for ten bit chip address);
	                                         - len - messageg length;
	                                         - *buf - pointer to message data */
	int rc;                        /* Return code */ 
};

struct mlnx_rdwr_smbus_ioctl_params {
	int bus;                       /* Bus id to which read or write request is to be sent */
	int addr_width;                /* Address width of i2c device */
	__u16 addr;	               /* slave address */
	__u16 flags;	               /* usually zero or I2C_CLIENT_PEC */
	__u8 read_write;               /* I2C_SMBUS_READ or I2C_SMBUS_WRITE */
	__u8 command;                  /* Byte interpreted by slave, for protocols which use such bytes */
	__u32 size;
	union i2c_smbus_data __user data; /* Data to be read or written */
	int rc;                        /* Return code */
};

struct mlnx_rdwr_smbus_ioctl_params_2 {
	int bus;                       /* Bus id to which read or write request is to be sent */
	int addr_width;                /* Address width of i2c device */
	__u16 addr;	               /* slave address */
	__u32 offset;                  /* Offset inside device address space */
	__u8 read_write;               /* I2C_SMBUS_READ or I2C_SMBUS_WRITE */
	__u32 datalen;                 /* Length of data */
	char __user data[SMBUS_BLOCK_MAX];  /* Data to be read or written */
	int rc;                        /* Return code */
};

struct mlnx_cpld_ioctl_params {
	int reg;
	unsigned char data;
};

struct mlnx_lpc_ioctl_params {
	__u16 base;                    /* base address of LPC bus */
	__u16 offset;                  /* offset within LPC device from which to start access */
	__u8 read_write;               /* read - 1, write 0 */
	__u8 datalen;                 /* Length of data */
	char __user data[LPCI2C_DATA_REG_SZ];
};

struct mlnx_lpc_kernel_params {
	__u16 base;                    /* base address of LPC bus */
	__u16 offset;                  /* offset within LPC device from which to start access */
	__u8 read_write;               /* read - 1, write 0 */
	__u8 datalen;                 /* Length of data */
	char* data;
};

struct irq_entry {
	__u8 dev_index;                /* Status, event or mask device index */
	__u8 range[2];                 /* Status, event or mask range -  from address to address */
};

#define RW_T_OPER(rw_oper)  ((rw_oper) & 0x01)
#define RW_T_SIZE(rw_oper)  ((rw_oper >> 4) & 0x0f)
struct irq_dev_signal {
	__u8 dev_index;                /* Device index */
	__u8 clear_addr;               /* Device clear interrupt address */
	__u8 clear_mask;               /* Device clear interrupt mask */
	__u8 rw_oper;                  /* LSB - in lower nibble defines device clear
					  interrupt method - 0 write, 1 - read.
					  higher nibble defines number of bytes that
					  should be accessed on clear. 
					  F.e. 0x21 means read 2 bytes at address clear_addr;
					       0x40 means write 4 bytes at address clear_addr,
					       all with clear_mask value. */
};

struct irq_sig_entry {
	__u8 signal_id;                /* Per-bit signal ids */
	struct irq_dev_signal dev_signal; /* Underlying device signal */
};

struct irq_config_entry {
	struct irq_entry e_status;     /* Status range -  from address to address */
	struct irq_entry e_event;      /* Event range -  from address to address */
	struct irq_entry e_mask;       /* Mask range -  from address to address */
	__u8 mask;                     /* Interrupt mask */
	__u8 level;                    /* Interrupt level */
	struct irq_sig_entry sig[8];   /* Per-bit signal ids in byte */
	__u16 profile;                 /* Entry profile*/
};

struct irq_data_entry {
	__u8 status;             /* Status map */
	__u8 event;              /* Event map */
};

struct mlnx_cpld_interrupt_set_params {
	union {
		struct cpld_mgmt_set {
			struct irq_config_entry s_entry[MAX_MGMT_IRQS];
		} mgmt_set;
		struct cpld_module_set {
			struct irq_config_entry s_entry[MAX_MOD_IRQS];
		} module_set;
		struct cpld_tor_set {
			struct irq_config_entry s_entry[MAX_TOR_IRQS];
		} tor_set;
	} u;
};

struct mlnx_cpld_interrupt_get_params {
	union {
		struct cpld_mgmt_get {
			struct irq_data_entry g_entry[MAX_MGMT_IRQS];
		} mgmt_get;
		struct cpld_module_get {
			struct irq_data_entry g_entry[MAX_MOD_IRQS];
		} module_get;
		struct cpld_tor_get {
			struct irq_data_entry g_entry[MAX_TOR_IRQS];
		} tor_get;
	} u;
};

struct dev_access_block {
	__u8 dev_index;
	union {
		struct mlnx_i2c_dev_params i2c_params;
		struct mlnx_lpc_dev_params lpc_params;
	} hwdev;
};

struct mlnx_cpld_subscribe_ioctl_params {
	int int_id;                    /* Interrupt line index */
	__u16 device_type;             /* Device attached to bus I2C_BUS_TYPE or to LPC_BUS_TYPE */
	__u8 card_type;                /* Card type: management, module, TOR */
	int usr_handle;                /* Upper user hanlde */
	__u32 int_level;               /* Interrupt's hierarchical level */
	int dev_id;                    /* Device id */
	irq_signal_e wake_on_signal;   /* For CARD_TYPE_MODULE or could be for some other aggregation - signal which is not ignored */
	struct dev_access_block hw_acc_tab[DEV_INT_CAP_MAX]; /* Table of interrupt capable devices */
	struct dev_access_block hw_term_acc_tab[DEV_TERM_INT_CAP_MAX]; /* Table of interrupt capable termination devices */
	struct bus_reset_block bus_rst;/* Bus physical reset block */
	__u16 flags;	               /* User specific flags */
	__u8 operation;                /* Subcribe or unsubcribe */
	__u8 irq_entry_num;            /* Number of entries in irq table */
	struct mlnx_cpld_interrupt_set_params irqs; /* irq mapping */
	int rc;                        /* Return code */
};

struct mlnx_cpld_set_ioctl_params {
	__u16 device_type;              /* device bus type, i2c, lpc, etc */
	int dev_id;                     /* Device id */
	union {
		struct mlnx_i2c_dev_params i2c_params;
		struct mlnx_lpc_dev_params lpc_params;
	} hwdev;
	__u16 flags;	                /* user specific flags */
	struct mlnx_cpld_interrupt_set_params irqs; /* irq mapping */
	int rc;                         /* Return code */
};

struct mlnx_cpld_get_data_ioctl_params {
	int dev_id;                     /* Device id */
	union {
		struct mlnx_i2c_dev_params i2c_params;
		struct mlnx_lpc_dev_params lpc_params;
	} hwdev;
	__u16 flags;	                /* user specific flags */
	__u16 device_type;              /* device bus type, i2c, lpc, etc */
	struct mlnx_cpld_interrupt_get_params get_irqs; /* irq mapping */
	int rc;                         /* Return code */
};

/* GPIO related ioctl parameters */
#define MLNX_GPIO_EXPORT    0x01
#define MLNX_GPIO_UNEXPORT  0x02
#define MLNX_GPIO_DIR       0x04
#define MLNX_WRITE_0        0x00
#define MLNX_WRITE_1        0x01
#define MLNX_WRITE_0_1      0x02
#define MLNX_WRITE_1_0      0x03
struct mlnx_gpio_ioctl_params {
	int pin;
	int dir;
	__u8 data;
	__u8 flags;
	int delay;   /* Delay in msec beetween two write accesses,
                        relevant only if data set to  MLNX_WRITE_0_1 or
                        MLNX_WRITE_1_0 for toggling 0-1 or 1-0 */
};

/* I2C SGMII related ioctl parameters */
struct mlnx_i2c_sgmii_ioctl_params {
	int bus;
	int phy_id;
	int phy_reg_addr;
	int datalen;
	unsigned char data[I2C_BLOCK_MAX];
};

/* I2C SWITCH related ioctl parameters */
struct mlnx_i2c_switchx_ioctl_params {
	int bus;
	int swictch_id;
	int phy_reg_addr;
	int datalen;
	unsigned char data[I2C_BLOCK_MAX];
};

struct i2c_switch_params {
	unsigned int switch_id;         /* Unique switch id */
	unsigned short reserved;
    unsigned char adap_num;         /* The number of the switches' legs (always 8 for PCA9548) */
    unsigned char deselect_on_exit; /* Dselect the switch legs after accessing it, if zero, if 1 - stay selected.
                                           There is no option set the port of swictch legs to 0, and part  to 1 */
	int i2c_dev_addr;               /* Physical i2c address of switch */
    int adap_id_array[LEGS_NUM];    /* Array of the buses numbers to lower edge direction of this i2c switch */
	unsigned char i2c_dev_type[I2C_NAME_SIZE]; /* Actually, driver name. For PCA9548 - string "pca9548" */
	char switch_name[SWITCH_NAME_SIZE]; /* User defined name. Can be empty. */
	int sel_reg_addr;               /* Relevant only for CPLD muxes. It's select register in CPLD
									   for this CPLD analog mux.
									   Should be negative for non cpld_mux */
};

struct mlnx_i2c_virt_bus_ioctl_params {
	unsigned char element_num;        /* Number of elememnts within request - could be up to MAX_SW_INREQ_NUM swicthes */
	unsigned short parent_bus_num;    /* Upper edge bus id to which switch or switches is (are) to be connected */
	unsigned char flag;               /* Flag for special purpose, depends on application */
	unsigned int upper_switch_id;     /* Id of upper edge swicth, or 0 if not exist */
	short status;                     /* Request status to be returned to user */
	struct  i2c_switch_params sw[MAX_SW_INREQ_NUM]; /* Swicth array */
};

struct i2c_dev_params {
	int i2c_dev_addr;                 /* Physical i2c address of device */
	char i2c_dev_type[I2C_NAME_SIZE]; /* Driver name (f.e. "switchx", "ds2161", "at24", etc */
	unsigned int user_handle;         /* Optional user handle for any specific purpose */
	void *platform_data;
};

struct mlnx_i2c_one_device_ioctl_params {
	unsigned short parent_bus_num;    /* Upper edge bus id to device to be connected */
	struct i2c_dev_params dev;        /* Device */
	unsigned short status;            /* Status of request to be returned back to caller */
};

struct mlnx_i2c_devices_ioctl_params {
	unsigned char element_num;        /* Number of elememnts within request - could be up to MAX_DEV_INREQ_NUM devices */
	unsigned char reserved;
	unsigned short parent_bus_num;    /* Upper edge bus id to which switch or switches is (are) to be connected */
	struct i2c_dev_params dev[MAX_DEV_INREQ_NUM]; /* Device array */
	unsigned short status;            /* Status of request to be returned back to caller */
};

struct mlnx_i2c_switch_ioctl_params {
	unsigned char element_num;        /* Number of elememnts within request - could be up to MAX_SW_INREQ_NUM swicthes */
	unsigned char flag;               /* Flag for special purpose, depends on application */
	unsigned short reserved;
	unsigned int upper_switch_id;     /* Id of upper edge swicth, or 0 if not exist */
	unsigned int sw[MAX_DEV_INREQ_NUM];  /* Unique switch id array */ 
};

struct mlnx_i2c_virt_bus_scan_ioctl_params {
	unsigned short parent_bus_num;    /* Bus id */
	unsigned short status;            /* Status of request to be returned back to caller */
	unsigned int presence[8];         /* Presence of device bit mask to be returned back: 
                                             0 - device is not presented, 
                                             1 - device is presented */
};

struct i2c_gpio_params {
	unsigned char i2c_dev_type[I2C_NAME_SIZE]; /* Actually, driver name. For i2c gpio muxer - string "gpio-i2cmux".*/
	int parent_bus_num;                 /* Parent I2C bus adapter number */
	int chip_id;                        /* Id of GPIO controller within chip */
	int base_bus_num;                   /* Base I2C bus number to number adapters from or zero for dynamic */
	int buses_num;                      /* Number of multiplexer positions (buses to instantiate) */
	int gpios_num;                      /* Number of GPIOs used to control MUX */
	unsigned buses[MAX_GPIO_NUM];       /* Array of bitmasks of GPIO settings (low/high) for each position */
#define GPIO_IN   0
#define GPIO_OUT  1
#define GPIO_LOW  2
#define GPIO_HIGH 3
	unsigned gpios[MAX_I2C_GPIO_BUSES]; /* Array of GPIO numbers used to control MUX */
	unsigned gpios_dir[MAX_I2C_GPIO_BUSES];   /* Array of GPIO directions for pins controlling MUX */
	unsigned gpios_value[MAX_I2C_GPIO_BUSES]; /* Array of GPIO initial values for pins controlling MUX */
	unsigned idle;                      /* Bitmask to write to MUX when idle (or GPIO_I2CMUX_NO_IDLE if not used) */
};

struct mlnx_gpio_i2c_ioctl_params {
	unsigned char element_num;        /* Number of elememnts within request - could be up to MAX_I2C_GPIO_INREQ_NUM */
	unsigned char reserved[3];
	struct i2c_gpio_params exp[MAX_I2C_GPIO_INREQ_NUM]; /* GPIO mux/expander array */
};

struct mlnx_gpio_i2c_del_ioctl_params {
	unsigned char element_num;        /* Number of elememnts within request - could be up to MAX_I2C_GPIO_INREQ_NUM */
	unsigned char reserved[3];
	int parent_bus_num[MAX_I2C_GPIO_INREQ_NUM];  /* Upper edge bus ids to which muxer/expander are connected */
};
#define MLNX_IOCTL_MAX_SIZE \
	max2( \
	max2(sizeof(struct mlnx_rdwr_i2c_ioctl_params), \
	max2(sizeof(struct mlnx_i2c_sgmii_ioctl_params), \
	max2(sizeof(struct mlnx_i2c_switchx_ioctl_params), \
	max2(sizeof(struct mlnx_cpld_ioctl_params), \
	max2(sizeof(struct mlnx_lpc_ioctl_params), \
	max2(sizeof(struct mlnx_gpio_ioctl_params), \
	max2(sizeof(struct mlnx_i2c_virt_bus_ioctl_params), \
	max2(sizeof(struct mlnx_i2c_switch_ioctl_params), \
	max2(sizeof(struct mlnx_i2c_devices_ioctl_params), \
	     sizeof(struct mlnx_i2c_virt_bus_scan_ioctl_params) \
	     ))))))))), \
	max2(sizeof(struct mlnx_gpio_i2c_ioctl_params), \
	max2(sizeof(struct mlnx_gpio_i2c_del_ioctl_params), \
	max2(sizeof(struct mlnx_i2c_one_device_ioctl_params), \
	max2(sizeof(struct mlnx_i2c_virt_bus_scan_ioctl_params), \
	max2(sizeof(struct mlnx_rdwr_smbus_ioctl_params), \
	max2(sizeof(struct mlnx_rdwr_smbus_ioctl_params_2), \
	max2(sizeof(struct mlnx_rdwr_vpd_i2c_ioctl_params), \
	max2(sizeof(struct mlnx_cpld_subscribe_ioctl_params), \
	max2(sizeof(struct mlnx_cpld_set_ioctl_params), \
	     sizeof(struct mlnx_cpld_get_data_ioctl_params) ))))))))) )
/* gcc: Internal error: Killed (program cc1)
	max2(sizeof(struct mlnx_cpld_get_data_ioctl_params), \
	     sizeof(struct mlnx_rdwr_vpd_i2c_ioctl_params) )))))))))))))))))))))
	     sizeof(struct mlnx_watchdog_config_ioctl_params) ))))))))))))))))))))))
	     sizeof(struct mlnx_watchdog_enable_disable_ioctl_params) )))))))))))))))))))))))
*/

struct mlnx_rdwr_vpd_i2c_ioctl_params {
	int bus;                                  /* Bus id to which read or write request is to be sent */
	unsigned addr;                            /* VPD slave address */
	unsigned flags;                           /* Access flag - 0 for write, 1 for read */
	unsigned datalen;                         /* Length of data is to be read or written */
	unsigned char databuf[2*I2C_CHUNK_MAX];   /* Data buffer */
	unsigned int offset;                      /* Offset inside VPD, when data is to be read or written */
	int addr_width;                           /* VPD internal address space */
	int bus_prot;                             /* Bus id, which is used for access protection */
	unsigned addr_prot;                       /* Device address, which is used for access protection */
	unsigned reg_prot;                        /* Device register offset, which is used for access protection */
	unsigned char get_prot;                   /* Value be written to get protection */
	unsigned char put_prot;                   /* Value be written to put protection */
	int limit;                                /* Access limit in msec */
	int border;                               /* Border limit, which could not be crossing by one transaction */
	int read_chunk_delay;                     /* Delay in msec between chunk reading */
	int write_chunk_delay;                    /* Delay in msec between chunk writing */
	int post_delay;                           /* Delay in msec before release protected bus */
	int rc;                                   /* Return code */
};

enum {
	IOCTL_MLNX_I2C_READ		           = _IOWR('A', 0x00, struct mlnx_rdwr_i2c_ioctl_params),
	IOCTL_MLNX_I2C_WRITE		           = _IOWR('A', 0x01, struct mlnx_rdwr_i2c_ioctl_params),
	IOCTL_MLNX_I2C_SGMII_READ	           = _IOWR('A', 0x02, struct mlnx_i2c_sgmii_ioctl_params),
	IOCTL_MLNX_I2C_SGMII_WRITE	           = _IOWR('A', 0x03, struct mlnx_i2c_sgmii_ioctl_params),
	IOCTL_MLNX_I2C_SWITCHX_READ	           = _IOWR('A', 0x04, struct mlnx_i2c_switchx_ioctl_params),
	IOCTL_MLNX_I2C_SWITCHX_WRITE	           = _IOWR('A', 0x05, struct mlnx_i2c_switchx_ioctl_params),
	IOCTL_MLNX_CPLD_RW		           = _IOWR('A', 0x06, struct mlnx_cpld_ioctl_params),
	IOCTL_MLNX_LPC_RW		           = _IOWR('A', 0x07, struct mlnx_lpc_ioctl_params),
	IOCTL_MLNX_GPIO_SET_DIR		           = _IOWR('A', 0x08, struct mlnx_gpio_ioctl_params),
	IOCTL_MLNX_GPIO_READ		           = _IOWR('A', 0x09, struct mlnx_gpio_ioctl_params),
	IOCTL_MLNX_GPIO_WRITE		           = _IOWR('A', 0x0A, struct mlnx_gpio_ioctl_params),
	IOCTL_MLNX_ADD_I2C_VIRTUAL_BUSES           = _IOWR('A', 0x0B, struct mlnx_i2c_virt_bus_ioctl_params),
	IOCTL_MLNX_DELETE_I2C_VIRTUAL_BUSES        = _IOWR('A', 0x0C, struct mlnx_i2c_switch_ioctl_params),
	IOCTL_MLNX_BLOCK_I2C_VIRTUAL_BUSES         = _IOWR('A', 0x0D, struct mlnx_i2c_switch_ioctl_params),
	IOCTL_MLNX_UNBLOCK_I2C_VIRTUAL_BUSES       = _IOWR('A', 0x0E, struct mlnx_i2c_switch_ioctl_params),
	IOCTL_MLNX_ADD_I2C_DEVICES_TO_VBUS         = _IOWR('A', 0x0F, struct mlnx_i2c_devices_ioctl_params),
	IOCTL_MLNX_DELETE_I2C_DEVICES_FROM_VBUS    = _IOWR('A', 0x10, struct mlnx_i2c_switch_ioctl_params),
	IOCTL_MLNX_SCAN_I2C_SWITCH_DEVICES         = _IOWR('A', 0x11, struct mlnx_i2c_virt_bus_scan_ioctl_params),
	IOCTL_MLNX_ADD_I2C_GPIO_VIRTUAL_BUSES      = _IOWR('A', 0x12, struct mlnx_gpio_i2c_ioctl_params),
	IOCTL_MLNX_DELETE_I2C_GPIO_VIRTUAL_BUSES   = _IOWR('A', 0x13, struct mlnx_gpio_i2c_del_ioctl_params),
	IOCTL_MLNX_ADD_I2C_ONE_DEVICE_TO_VBUS      = _IOWR('A', 0x14, struct mlnx_i2c_one_device_ioctl_params),
	IOCTL_MLNX_DELETE_I2C_ONE_DEVICE_FROM_VBUS = _IOWR('A', 0x15, struct mlnx_i2c_one_device_ioctl_params),
	IOCTL_MLNX_RESET_I2C_VIRTUAL_BUSES_AND_DEVICES = _IOWR('A', 0x16, struct mlnx_i2c_switch_ioctl_params),
	IOCTL_MLNX_TEST_I2C_VBUS                   = _IOWR('A', 0x17, struct mlnx_i2c_virt_bus_scan_ioctl_params),
	IOCTL_MLNX_TEST_I2C_DEV                    = _IOWR('A', 0x18, struct mlnx_i2c_one_device_ioctl_params),
	IOCTL_MLNX_SMBUS_READ                      = _IOWR('A', 0x19, struct mlnx_rdwr_smbus_ioctl_params),
	IOCTL_MLNX_SMBUS_WRITE                     = _IOWR('A', 0x1A, struct mlnx_rdwr_smbus_ioctl_params),
	IOCTL_MLNX_SMBUS_RW                        = _IOWR('A', 0x1B, struct mlnx_rdwr_smbus_ioctl_params_2),
	IOCTL_MLNX_CPLD_IRQ_SUBSCRIBE              = _IOWR('A', 0x1C, struct mlnx_cpld_subscribe_ioctl_params),
	IOCTL_MLNX_CPLD_IRQ_UNSUBSCRIBE            = _IOWR('A', 0x1D, struct mlnx_cpld_subscribe_ioctl_params),
	IOCTL_MLNX_CPLD_IRQ_SET                    = _IOWR('A', 0x1E, struct mlnx_cpld_set_ioctl_params),
	IOCTL_MLNX_CPLD_IRQ_GET_DATA               = _IOWR('A', 0x1F, struct mlnx_cpld_get_data_ioctl_params),
        IOCTL_MLNX_VPD_READ                        = _IOWR('A', 0x20, struct mlnx_rdwr_vpd_i2c_ioctl_params),
        IOCTL_MLNX_VPD_WRITE                       = _IOWR('A', 0x21, struct mlnx_rdwr_vpd_i2c_ioctl_params),
	IOCTL_MLNX_WATCHDOG_CONFIG                 = _IOWR('A', 0x22, struct mlnx_watchdog_config_ioctl_params),
	IOCTL_MLNX_WATCHDOG_ENA_DIS                = _IOWR('A', 0x23, struct mlnx_watchdog_enable_disable_ioctl_params),
	IOCTL_MLNX_CPLD_CONFIG_LED                 = _IOWR('A', 0x24, struct mlnx_cpld_leds_config_ioctl_params),
	IOCTL_MLNX_ASIC_CONFIG_FAN                 = _IOWR('A', 0x25, struct mlnx_asic_fan_config_ioctl_params),
	IOCTL_MLNX_ASIC_CONFIG_TEMP                = _IOWR('A', 0x26, struct mlnx_asic_temp_sensor_config_ioctl_params),
};

void mlnx_rw_lpc_kernel(struct mlnx_lpc_kernel_params *params);

#endif /* __MLNX_IF_H_ */
