/**
 *
 * Copyright (C) 2013-2014, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */
#ifndef __MLNX_LOG_H_
#define __MLNX_LOG_H_

extern int *mlnx_debug_level_symbol;
#define MLNX_LOG_MIN            1
#define MLNX_LOG_MAX            3
#define MLNX_LOG_STEP           (MLNX_LOG_MAX - MLNX_LOG_MIN)
#define MLNX_LOG_WD_MIN         (MLNX_LOG_MAX + 1)                          /*  4 */
#define MLNX_LOG_WD_MAX         (MLNX_LOG_WD_MIN + MLNX_LOG_STEP)           /*  6 */
#define MLNX_LOG_IRQ_MIN        (MLNX_LOG_WD_MAX + 1)                       /*  7 */
#define MLNX_LOG_IRQ_MAX        (MLNX_LOG_IRQ_MIN + MLNX_LOG_STEP)          /*  9 */
#define MLNX_LOG_SXI2C_MIN      (MLNX_LOG_IRQ_MAX + 1)                      /* 10 */
#define MLNX_LOG_SXI2C_MAX      (MLNX_LOG_SXI2C_MIN + MLNX_LOG_STEP)        /* 12 */
#define MLNX_LOG_MELLAGGRA_MIN  (MLNX_LOG_SXI2C_MAX + 1)                    /* 13 */
#define MLNX_LOG_MELLAGGRA_MAX  (MLNX_LOG_MELLAGGRA_MIN + MLNX_LOG_STEP)    /* 15 */
#define MLNX_LOG_MEL_IRQ_MIN    (MLNX_LOG_MELLAGGRA_MAX + 1)                /* 16 */
#define MLNX_LOG_MEL_IRQ_MAX    (MLNX_LOG_MEL_IRQ_MIN + MLNX_LOG_STEP)      /* 18 */
#define MLNX_LOG_BSP_MIN        (MLNX_LOG_MEL_IRQ_MAX + 1)                  /* 19 */
#define MLNX_LOG_BSP_MAX        (MLNX_LOG_BSP_MIN + MLNX_LOG_STEP)          /* 21 */

#define mlnx_log(level, max, format, args...)  if (mlnx_debug_level_symbol) { \
						if ( (((*mlnx_debug_level_symbol) > MLNX_LOG_MIN) && \
						      ((*mlnx_debug_level_symbol) >= ((level)%MLNX_LOG_MAX)) && \
						      ((*mlnx_debug_level_symbol) <= MLNX_LOG_MAX)) || \
						     (((*mlnx_debug_level_symbol) >= (level)) && \
						      ((*mlnx_debug_level_symbol) <= (max)))) { \
							printk(KERN_INFO "mlnx_log: " format, ## args); \
						} \
					  }

static inline int hexalpha_to_int(int c)
{
	char hexalpha[] = "aAbBcCdDeEfF";
	int i;
	int inp = 0;

	for (i = 0; inp == 0 && hexalpha[i] != '\0'; i++) {
		if(hexalpha[i] == c)
			inp = 10 + (i / 2);
	}
	return inp;
}

static inline unsigned int htoi(const char s[])
{
	unsigned int inp = 0;
	int i = 0;
	int valid = 1;
	int hexit;

	if(s[i] == '0') {
		++i;
		if(s[i] == 'x' || s[i] == 'X')
			++i;
	}
	while(valid && s[i] != '\0') {
		inp = inp * 16;
		if(s[i] >= '0' && s[i] <= '9') {
			inp = inp + (s[i] - '0');
		}
		else {
			hexit = hexalpha_to_int(s[i]);
			if(hexit == 0)
				valid = 0;
			else
				inp = inp + hexit;
		}
		++i;
	}
	if(!valid)
		inp = 0;
	return inp;
}

static inline void hex_dump(char *data, int size, int min, int max)
{
	int i; // index in data...
	int j; // index in line...
	char temp[8];
	char buffer[128];
	char *ascii;
	 
	memset(buffer, 0, 128);	 
	mlnx_log(3, 3, "        +0          +4          +8          +c            0   4   8   c   \n"); 
	/* Hex portion of the line is 8 (the padding) + 3 * 16 = 52 chars long
	   with added another four bytes padding and place the ASCII version */
	ascii = buffer + 58;
	memset(buffer, ' ', 58 + 16);
	buffer[58 + 16] = '\n';
	buffer[58 + 17] = '\0';
	buffer[0] = '+';
	buffer[1] = '0';
	buffer[2] = '0';
	buffer[3] = '0';
	buffer[4] = '0';
	for (i = 0, j = 0; i < size; i++, j++) {
		if (j == 16) {
			mlnx_log(3, 3, "%s", buffer);
			memset(buffer, ' ', 58 + 16); 
			sprintf(temp, "+%04x", i);
			memcpy(buffer, temp, 5); 
			j = 0;
		}
		sprintf(temp, "%02x", 0xff & data[i]);
		memcpy(buffer + 8 + (j * 3), temp, 2);
		if ((data[i] > 31) && (data[i] < 127))
			ascii[j] = data[i];
		else
			ascii[j] = '.';
	} 
	if (j != 0)
		mlnx_log(min, max, "%s", buffer);
}

#endif /*__MLNX_LOG_H_*/
