/**
 *
 * Copyright (C) 2013-2014, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */
#ifndef __MLNX_WATCHDOG_H_
#define __MLNX_WATCHDOG_H_

#ifndef __KERNEL__
#define __user 
#endif

#include <linux/watchdog.h>
#include "mlnx_hwdev.h"

#define MLNX_WATCHDOG_CONFIG     0x22
#define MLNX_WATCHDOG_ENA_DIS    0x23
#define MLNX_WDIOC_GETSUPPORT       WDIOC_GETSUPPORT
#define MLNX_WDIOC_GETSTATUS        WDIOC_GETSTATUS
#define MLNX_WDIOC_GETBOOTSTATUS    WDIOC_GETBOOTSTATUS
#define MLNX_WDIOC_KEEPALIVE        WDIOC_KEEPALIVE
#define MLNX_WDIOC_SETTIMEOUT       WDIOC_SETTIMEOUT
#define MLNX_WDIOC_GETTIMEOUT       WDIOC_GETTIMEOUT

#define MLNX_WD_NAME              "mlnx_watchdog"
#define MLNX_WD1_PATH             "/dev/mlnx_watchdog1"
#define MLNX_WD2_PATH             "/dev/mlnx_watchdog2"
#define MLNX_WD3_PATH             "/dev/mlnx_watchdog3"
#define MLNX_WD4_PATH             "/dev/mlnx_watchdog4"

typedef enum wd_entry_type {
	_wd_1,               // Watchdog 1
	_wd_2,               // Watchdog 2
	_wd_3,               // Watchdog 3
	_wd_4,               // Watchdog 4
} wd_entry_type_e;
#define MAX_MGMT_WD   (_wd_4 - _wd_1 + 1)

typedef enum wd_obj {
        _wd1_                     = 0x0001,
        _wd2_                     = 0x0002,
        _wd3_                     = 0x0004,
        _wd4_                     = 0x0008,
        _wd_all_                  = 0x000f,
} wd_obj_e;

typedef enum wd_actions {
        _none_action              = 0x00,
        _act_reset_local          = 0x01,
        _act_int_to_local         = 0x02,
        _act_int_to_far           = 0x04,
        _act_power_off_local      = 0x08,
        _act_increment_cntr       = 0x80,
        _act_all                  = 0xff,
} wd_actions_e;

typedef enum wd_clear {
        _none_clear               = 0x00,
        _clear_timer              = 0x01,
        _clear_counter            = 0x02,
        _clear_all                = 0x03,
} wd_clear_e;

typedef enum wd_signals {
        _none_wd_sig              = 0x0000,
        _wd1_timer_clear          = 0x0001,
        _wd2_timer_clear          = 0x0002,
        _wd3_timer_clear          = 0x0003,
        _wd4_timer_clear          = 0x0004,
        _wd1_counter_clear        = 0x0005,
        _wd2_counter_clear        = 0x0006,
        _wd3_counter_clear        = 0x0007,
        _wd4_counter_clear        = 0x0008,
        _wd_reset_local           = 0x0009,
        _wd_int_to_local          = 0x000a,
        _wd_int_to_far            = 0x000b,
        _wd_power_off_local       = 0x000c,
        _wd_increment_cntr        = 0x000d,
} wd_signals_e;

#define _WD_CLEAR_MASK              0x000000ff;
#define _WD_ACTION_MASK             0x0000ff00
#define _WD_CLEAR_GET(v)            ((v) & _WD_CLEAR_MASK)
#define _WD_ACTION_GET(v)           (((v) & _WD_ACTION_MASK) >> 8)
typedef enum wd_entry_mask {
        _wd1_timer_clear_m        = 0x00000001,
        _wd2_timer_clear_m        = 0x00000002,
        _wd3_timer_clear_m        = 0x00000004,
        _wd4_timer_clear_m        = 0x00000008,
        _wd1_counter_clear_m      = 0x00000010,
        _wd2_counter_clear_m      = 0x00000020,
        _wd3_counter_clear_m      = 0x00000040,
        _wd4_counter_clear_m      = 0x00000080,
        _wd_reset_local_m         = 0x00000100,
        _wd_int_to_local_m        = 0x00000200,
        _wd_int_to_far_m          = 0x00000400,
        _wd_power_off_local_m     = 0x00000800,
        _wd_increment_cntr_m      = 0x00008000,
} wd_entry_mask_e;

typedef enum wd_sw_actions {
        _wd_sw_no_action          = 0x00,
        _wd_sw_notify             = 0x01,
        _wd_sw_reboot             = 0x02,
        _wd_sw_log                = 0x04,
        _wd_sw_all                = 0x07,
} wd_sw_actions_e;

struct wd_sig_entry {
	__u8 signal_id;                /* Per-bit signal ids */
	__u8 under_signal;             /* Per-bit underlying signal */
};

struct watchdog_sub_entry {
	__u8 wd_timer_lsb;             /* Watchdog timer address */
	__u8 wd_action;                /* Watchdog action address */
	__u8 wd_counter;               /* Watchdog counter address */
	struct wd_sig_entry sig[8];    /* Per-bit signal ids in byte */
};

struct watchdog_config_entry {
	__u8 dev_index;                /* Watchdog device index */
	__u8 wd_clear;                 /* Watchdog clear address */
	__u8 wd_clear_wp;              /* Watchdog clear write protection address */
	struct wd_sig_entry sig[8];    /* Per-bit signal ids in byte */
	struct watchdog_sub_entry se_wd[MAX_MGMT_WD];
};

#define MAX_WD_TIMER_VALUE 0x0f
struct mlnx_watchdog_ctrl {
	__u8 inc_thresh;                /* Increment threshold - in case watchdog is not
					   configured for reset or power off (_wd_reset_local
					   or _wd_power_off_local), inc_thresh if set to non-zero
					   value, defines the threshold value for increment counter.
					   After this threshold value is reached, system will be
					   forced for emergency restart.*/
	__u8 timer_value;               /* Per watchdog timer value - provided as a power of 2 in msec */
	__u8 hw_action;                 /* HW actions to be allowed for watchdog:
					   as defined in wd_actions_e, combination allowed */
	__u8 sw_action;                 /* SW actions to be allowed for watchdog:
					   as defined in wd_sw_action_e, combination allowed */
	__u8 force_sw_reboot;           /* If force_reboot is set it will cause an immediate system
                                           reset in case of software time is set and reset criteria is
                                           matched on its expiration. */
};

struct mlnx_watchdog_config_ioctl_params {
	int dev_id;                     /* Device id */
	union {
		struct mlnx_i2c_dev_params i2c_params;
		struct mlnx_lpc_dev_params lpc_params;
	} hwdev;
	__u16 flags;	                /* user specific flags */
	__u16 device_type;              /* device bus type, i2c, lpc, etc */
	struct watchdog_config_entry wd_entry;
	struct mlnx_watchdog_ctrl ctrl[MAX_MGMT_WD]; /* Per watchdog control entry */
	int rc;                         /* Return code */
};

struct mlnx_watchdog_enable_disable_ioctl_params {
	wd_obj_e wd;                    /* Watchdogs to be enabled or disabled */
	__u8 ena;                       /* Enable - 1, disable - 0 */
	__u8 enable_delay;              /* Delay to be performed before watchdog is enabled for real */
	int rc;                         /* Return code */
};

extern int (*mlnx_watchdog_config_symbol)(struct mlnx_watchdog_config_ioctl_params *wd_config_params,
		rw_bus_access_func rw_func);
extern int (*mlnx_watchdog_enable_disable_symbol)(struct mlnx_watchdog_enable_disable_ioctl_params *wd_ed_params);
extern int (*mlnx_watchdog_timer_clear_symbol)(wd_obj_e wd_id);
extern int (*mlnx_watchdog_counter_clear_symbol)(wd_obj_e wd_id);

#endif /*__MLNX_WATCHDOG_H_*/
